package com.engine.salary.remote.cbs8.client;

import com.engine.salary.exception.CBS8RunTimeException;
import com.engine.salary.remote.cbs8.request.GetTransactionDetailRequest;
import com.engine.salary.remote.cbs8.response.GetTransactionDetailResponse;
import com.engine.salary.remote.cbs8.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

@Slf4j
public class AccountManagementClient extends CBS8BaseClient {

    public GetTransactionDetailResponse transactionDetailQuery(GetTransactionDetailRequest requestParam) throws IOException {
        String url = host + "/openapi/account/openapi/v1/transaction-detail/query";


        CloseableHttpClient client = HttpClients.custom()
                // 禁止HttpClient自动解压缩
                .disableContentCompression()
                .build();

        String requestData = JsonUtil.toJsonString(requestParam);
        log.info("获取cbs交易参数，" + url + "\n" + requestData);

        HttpPost httpPost = setupRequest(url, requestData);
        try (CloseableHttpResponse response = client.execute(httpPost)) {
            byte[] finalResponseData = handleResponse(response);
            String req = new String(finalResponseData, StandardCharsets.UTF_8);
            GetTransactionDetailResponse getTransactionDetailResponse = JsonUtil.parseBean(req, GetTransactionDetailResponse.class);
            log.info("获取cbs交易结果，" + "\n" + JsonUtil.toJsonString(getTransactionDetailResponse));

            if (Objects.isNull(getTransactionDetailResponse)) {
                throw new CBS8RunTimeException("服务异常");
            }
            if (!"0".equals(getTransactionDetailResponse.getCode())) {
                throw new CBS8RunTimeException(getTransactionDetailResponse.getMsg());
            }
            if (getTransactionDetailResponse.getData() == null) {
                throw new CBS8RunTimeException("未获取数据");
            }

            return getTransactionDetailResponse;

        } catch (IOException ignored) {
            log.error("网络连接失败或超时！",ignored);
            throw new CBS8RunTimeException("网络连接失败或超时！");
        } finally {
            client.close();
        }
    }

}
