package com.engine.salary.remote.cbs8.response;

import com.alibaba.fastjson.JSON;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Data
@EqualsAndHashCode(callSuper = true)
public class GetTransactionDetailResponse extends CBS8BaseResponse {
    /**
     * 返回数据
     */
    private Body data;

    @Data
    @EqualsAndHashCode(callSuper = true)
    public static class Body extends CBS8PageInfo{
        /**
         * 明细列表
         */
        private List<Detail> list;
    }

    @Data
    public static class Detail {
        /**
         * 银行账号
         */
        private String accountNo;
        /**
         * 账户名称
         */
        private String accountName;
        /**
         * 银行类型    银行类型见附录4.1.5
         */
        private String bankType;
        /**
         * 开户行名称
         */
        private String openBank;
        /**
         * 交易日期    时间戳
         */
        private Date bankTransactionDate;
        /**
         * 银行流水号
         */
        private String bankSerialNumber;
        /**
         * 交易流水号
         */
        private Long transactionSerialNumber;
        /**
         * 币种    币种枚举见附录4.1.1
         */
        private String currency;
        /**
         * 借贷类型    1-借；2-贷
         */
        private String loanType;

        /**
         * 发生额
         */
        private BigDecimal incurredAmount;

        /**
         * 交易后余额
         */
        private BigDecimal accountBalance;
        /**
         * 用途
         */
        private String purpose;
        /**
         * 摘要
         */
        private String digest;
        /**
         * 对方账号
         */
        private String oppositeAccount;
        /**
         * 对方户名
         */
        private String oppositeName;
        /**
         * 对方开户行
         */
        private String oppositeOpeningBank;
        /**
         * 关联客户号
         */
        private String associatedCustomerNumber;
        /**
         * 客商编号    客户在公共设置>基础信息>客商管理登记的客商编码，例如C002-XX供应链管理。此处返回C002。
         */
        private String merchantNumber;
        /**
         * 客商名称    客户在公共设置>基础信息>客商管理登记的客商名称，例如C002-XX供应链管理。此处返回XX供应链管理。
         */
        private String merchantName;
        /**
         * 起息日    时间戳
         */
        private Date valueDate;
        /**
         * 交易代码    见附录枚举4.1.8
         */
        private String transactionCode;
        /**
         * 款项性质匹配标记    1-未匹配；2-手工处理；3-系统处理；4-匹配失败
         */
        private String paymentNatureFlag;
        /**
         * 款项性质代码    客户在公共设置>基础信息>款项性质查询登记的款项性质，例如A001-往来款，此处返回A001
         */
        private String paymentNature;
        /**
         * 明细来源    B：银行，即银行直联 U：用户，即手工/ERP接口导入
         */
        private String detailSource;
        /**
         * 明细类型    1-当日明细  2-历史明细 与明细日期无关，仅标识数据来源银行的不同接口。
         */
        private String detailType;
        /**
         * 账户状态    0-正常，1-销户
         */
        private String accountStatus;
        /**
         * 账户性质    客户在公共设置>基础信息>账户性质查询自定义内容，例如AA-综合户，此处返回值“AA”
         */
        private String accountNature;
        /**
         * 对账码    对账码，是CBS8系统通过一定规则匹配交易明细与回单或交易明细与支付成功后生成的对账码
         */
        private String checkCode;
        /**
         * 单位编码    客户在公共设置>基础信息>组织机构维护，例如0001-XX科技有限公司，此处返回0001
         */
        private String unitCode;
        /**
         * 单位名称
         */
        private String unitName;
        /**
         * 备注
         */
        private String remark;
        /**
         * ERP业务参考号    此字段为支付接口的referenceNum（业务参考号）
         */
        private String erpSerialNumber;
        /**
         * 支付申请备注1
         */
        private String PayApplyRemark1;
        /**
         * 支付申请备注2
         */
        private String PayApplyRemark2;
        /**
         * 支付申请备注3
         */
        private String PayApplyRemark3;
        /**
         * 交易识别码
         */
        private String corporateIdentityCode;
        /**
         * 预留字段1
         */
        private String reserveField1;
        /**
         * 预留字段2
         */
        private String reserveField2;
        /**
         * 预留字段3
         */
        private String reserveField3;
        /**
         * 预留字段4
         */
        private String reserveField4;
        /**
         * 扩展字段信息    fieldKey：字段KEY title : 标题  content : 字段内容
         */
        private List<JSON> extensionMsg;
        /**
         * 附言
         */
        private String postscript;
        /**
         * 凭证编号
         */
        private String voucherCode;
        /**
         * 记账日期
         */
        private Date bookingDate;
        /**
         * 银行个性化字段1    银行个性化字段见附录4.1.20
         */
        private String personalizeField1;
        /**
         * 银行个性化字段2    银行个性化字段见附录4.1.20
         */
        private String personalizeField2;
        /**
         * 银行个性化字段3    银行个性化字段见附录4.1.20
         */
        private String personalizeField3;
        /**
         * 银行个性化字段4    银行个性化字段见附录4.1.20
         */
        private String personalizeField4;
        /**
         * 银行个性化字段5    银行个性化字段见附录4.1.20
         */
        private String personalizeField5;
        /**
         * 虚拟户账号
         */
        private String virtualAccount;
        /**
         * 虚拟户账号名称
         */
        private String virtualAccountName;
        /**
         * 交易类型
         */
        private String protocolType;
        /**
         * 交易类型名称
         */
        private String protocolTypeName;
        /**
         * 母/子公司账号
         */
        private String parentSubCompanyAccount;
        /**
         * 母/子公司名称母/子公司名称
         */
        private String parentSubCompanyName;
        /**
         * 通过9091->公共设置->业务配置->
         * 通用参数管理->账户管理->erp_trans_detail_mark_return_flag(交易明细查询ERP接口是否返回关联业务表字段)设置可开启字段返回，参数关闭时字段为null
         * id:关联业务id
         * bizCode:关联业务(见附录4.1.22)
         * bizFlag:关联业务标记
         * (0-手工标记
         * 1-按规则标记
         * 2-模块关联标记
         * 3-ERP接口标记)
         * moduleCode:关联模块
         * menuName:关联菜单
         * bizOrderNo:业务编号
         */
        private List<JSON> markingList;
    }
}
